/***************************************************************************
*   Copyright (C) 2003 by Hans Karlsson                                   *
*   karlsson.h@home.se                                                      *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/
#include "xmmssensor.h"
XMMSSensor::XMMSSensor( int interval, QString encoding ) : Sensor( interval )
{
     if( encoding != "" )
    {
        codec = QTextCodec::codecForName( encoding );
        if ( codec == 0)
            codec = QTextCodec::codecForLocale();
    }
    else{
        codec = QTextCodec::codecForLocale();
    }

}
XMMSSensor::~XMMSSensor()
{}

int XMMSSensor::getW(){
  KSimpleConfig conf("karambarc",true);
  conf.setGroup("XMMS");
  w = conf.readNumEntry("CoverWidth",64);
   return w;
}

int XMMSSensor::getH(){
  KSimpleConfig conf("karambarc",true);
  conf.setGroup("XMMS");
  h = conf.readNumEntry("CoverHeight",64);
  return h;
}

QString XMMSSensor::setCover(){
  int w = getW();
  int h = getH();
  QImage previmg;
  fp = locateLocal("tmp", "cover.png");
  previmg.load(fp);
  if(previmg.text("Title") != title || previmg.width() != w || previmg.height() != h){ //Verify song change to limit CPU usage
    QImage img;
    QImage tmpimg;
    img.load(rcover);
    if(img.width() >= w || img.height() >= h){ //scale image if necessary
      tmpimg = img.scale(h, w, QImage::ScaleMin);
       tmpimg.setText("Title", 0, title); //add Title in the image text for cache usage
    }
    tmpimg.save(fp, "PNG", 0);
  }
  return fp;
}

bool XMMSSensor::id3cover(QString file){
  int w = getW();
  int h = getH();
  QImage previmg;
  fp = locateLocal("tmp", "cover.png");
  previmg.load(fp);
  if(previmg.text("Title") != title || previmg.width() != w || previmg.height() != h){ //Verify song change to limit CPU usage
    myTag = new ID3_Tag();
    myTag->Link(file, ID3TT_ALL);
    ID3_Frame* myFrame = myTag->Find(ID3FID_PICTURE);// Check if id3 have PICTURE field
     if (NULL != myFrame){
      //qDebug("id3 have picture ");
      ID3_Field* myField = myFrame->GetField(ID3FN_DATA);
       const uchar* data = myField->GetRawBinary();
       size_t size = myField->Size();
      QImage img;
      QImage tmpimg;
      img.loadFromData (data, size);
      if(img.width() >= w || img.height() >= h){ //scale image if necessary
        tmpimg = img.scale(h, w, QImage::ScaleMin);
        tmpimg.setText("Title", 0, title); //add Title in the image text for cache usage
      }
      tmpimg.save(fp, "PNG", 0);
    }
    else{
      //qDebug("id3 don't have picture");
      return false;
    }
    return true;
  }
  return true;
}

void XMMSSensor::update()
{
    QString format;
    QString imgsize;
    SensorParams *sp;
    Meter *meter;
    QObjectListIt it( *objList );

#ifdef HAVE_XMMS

    int pos;
    int songLength = 0;
    int currentTime = 0;
    bool isPlaying = false;
    bool isRunning = xmms_remote_is_running(0);
    bool id3c = false;

    if( isRunning )
    {
        isPlaying = xmms_remote_is_playing(0);
        pos = xmms_remote_get_playlist_pos(0);
        //qDebug("unicode start");
        title = codec->toUnicode( QCString( xmms_remote_get_playlist_title( 0, pos ) )  );
        file = codec->toUnicode( QCString( xmms_remote_get_playlist_file( 0, pos ) )  ); //get the path and filename
        //qDebug("unicode end");
        if( title == "" ){
          if(file != ""){
            title = file;
          }
          else{
            title = "XMMS";
          }
        }
        if(file.section('.', -1) == "mp3" || file.section('.', -1) == "MP3"){ // Check if file is an mp3
          id3c = id3cover(file);
          if(id3c == true){
            cover = locateLocal("tmp", "cover.png");
          }
        }
        if(id3c == false){ // File is not an mp3 or don't have id3 PICTURE field
          QUrl url(file); //make QUrl from QString
          dir = url.dirPath (); //resolve the absolute path
          if(QFile::exists (dir + "/cover.png")){
            rcover = dir + "/cover.png";
            cover = setCover();
          }
          else if(QFile::exists (dir + "/cover.jpg")){
            rcover = dir + "/cover.jpg";
            cover = setCover();
          }
          else if(QFile::exists (dir + "/cover.jpeg")){
            rcover = dir + "/cover.jpeg";
             cover = setCover();
         }
          else{
            cover = codec->toUnicode("pics/nocover.png");
          }
         }

        //qDebug(cover);
        //qDebug(title);
        songLength = xmms_remote_get_playlist_time( 0, pos );
        currentTime = xmms_remote_get_output_time( 0 );
    }
#endif // HAVE_XMMS

    while (it != 0)
    {
        sp = (SensorParams*)(*it);
        meter = sp->getMeter();

#ifdef HAVE_XMMS

        if( isRunning )
        {

            format = sp->getParam("FORMAT");
            QString click = sp->getParam("ONCLICK");
            QString tx = sp->getParam("X");
            QString ty = sp->getParam("Y");

            if (format.length() == 0 )
            {
                format = "%title %time / %length";
            }

            if( format == "%ms" )
            {
                meter->setMax( songLength );
                meter->setValue( currentTime );
            }
            else

                if ( format == "%full" )
                {
                    meter->setValue( 1 );
                }
                else
                {


                    format.replace( QRegExp("%title", false), title );
                    if(format == "%cover"){
                      click.replace( QRegExp("%cover", false), cover );
                      meter->setValue(click);
                      format.replace( QRegExp("%cover", false), cover );
                   }

                    format.replace( QRegExp("%length", false), QTime( 0,0,0 ).
                                    addMSecs( songLength )
                                    .toString( "h:mm:ss" ) );

                    format.replace( QRegExp("%time", false), QTime( 0,0,0 ).
                                    addMSecs( currentTime )
                                    .toString( "h:mm:ss" ) );

                    if( isPlaying  )
                    {
                        format.replace( QRegExp("%remain", false), QTime( 0,0,0 ).
                                        addMSecs( songLength )
                                        .addMSecs(-currentTime )
                                        .toString( "h:mm:ss" ) );
                    }

                    else
                    {
                        format.replace( QRegExp("%remain", false), QTime( 0,0,0 ).toString("h:mm:ss" ) );
                    }
                    meter->setValue(format);
                }
       }
        else
#endif // HAVE_XMMS

        {
            meter->setValue("");
        }
        ++it;

    }

}

void XMMSSensor::setMaxValue( SensorParams *sp)
{
    Meter *meter;
    meter = sp->getMeter();
    QString f;
    f = sp->getParam("FORMAT");

    if ( f == "%full" )
        meter->setMax( 1 );

}
